from pyglet.gl import *
from vector import *
import math

PI = 3.14159265358979323846
INTERVAL = (1000.0 / 60.0) / 1000.0
MIN_DIST = 3.0
MAX_DIST = 30.0

class Camera(object):
	def __init__(self, vx=0.0, vy=0.0, vz=0.0,
	             dist=10.0, lat=30.0, lon=0.0):
		self.dist = dist
		self.latitude = lat
		self.longitude = lon
		self.position = Vector3()
		self.viewpoint = Vector3()
		self.set_viewpoint(vx, vy, vz)
		
	def set_viewpoint(self, x, y, z):
		self.viewpoint[0] = x
		self.viewpoint[1] = y
		self.viewpoint[2] = z
		self.position[0] = (x + self.dist *
		                    math.sin(self.longitude * PI / 180.0) *
							math.cos(self.latitude * PI / 180.0))
		self.position[1] = (y + self.dist *
		                    math.sin(self.latitude * PI / 180.0))
		self.position[2] = (z + self.dist *
		                    math.cos(self.longitude * PI / 180.0) *
							math.cos(self.latitude * PI / 180.0))

	def set_distance(self, d):
		self.dist = d
		
	def set_longitude(self, lon):
		self.longitude = lon
		
	def set_latitude (self, lat):
		self.latitude = lat
		
	def get_position(self):
		return self.position
		
	def get_viewpoint(self):
		return self.viewpoint
	
	def get_distance(self):
		return self.dist
		
	def get_longitude(self):
		return self.longitude
		
	def get_latitude(self):
		return self.latitude
		
	def zoom_in(self):
		if self.dist >= MIN_DIST:
			self.dist -= 0.1
			
	def zoom_out(self):
		if self.dist <= MAX_DIST:
			self.dist += 0.1
		
	def rotate_left(self):
		self.longitude -= 120.0 * INTERVAL
		
	def rotate_right(self):
		self.longitude += 120.0 * INTERVAL
		
	def rotate_up(self):
		self.latitude += 120 * INTERVAL
		if self.latitude > 89.0:
			self.latitude = 89.0
		
	def rotate_down(self):
		self.latitude -= 120 * INTERVAL
		if self.latitude < 0.0:
			self.latitude = 0.0
		
	def update_position(self, v):				
		np = Vector3()
		np[0] = (v[0] + self.dist *
		         math.sin(self.longitude * PI / 180.0) *
				 math.cos(self.latitude * PI / 180.0))
		np[1] = (v[1] + self.dist *
				 math.sin(self.latitude * PI / 180.0))
		np[2] = (v[2] + self.dist *
		         math.cos(self.longitude * PI / 180.0) *
				 math.cos(self.latitude * PI / 180.0))				
		diff = np - self.position
		error = diff.length()
		diff.normalize()
		diff.scale(5.0 * INTERVAL * error)
		self.position = (self.position + diff)		
			
	def move(self, v):
		self.update_position(v)					  
		diff = v - self.viewpoint
		error = diff.length()
		diff.normalize()
		self.viewpoint = (self.viewpoint +
		                  diff.scale(5.0 * INTERVAL * error))

	def update(self):
		self.update_position(self.viewpoint)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		gluLookAt(self.position.x, self.position.y,
				  self.position.z, self.viewpoint.x,
				  self.viewpoint.y, self.viewpoint.z,
				  0.0, 1.0, 0.0)

