from camera import *
from cat import *
from map import *
from parsefile import *
from pyglet.gl import *
from pyglet.window import key
import pyglet
import random
import math
import sys

pyglet.options['debug_gl'] = False
def zero_errcheck(result, func, arguments):
	return result
pyglet.gl.lib.errcheck = zero_errcheck

GAME_STATE_MAIN_MENU = 0
GAME_STATE_INGAME_MENU = 1
GAME_STATE_LOAD_LEVEL = 2
GAME_STATE_PLAY_LEVEL = 3
GAME_STATE_LEVEL_DONE = 4
GAME_STATE_FAIL = 5
GAME_STATE_GAME_DONE = 6
GAME_STATE_INSTRUCTIONS = 7
GAME_STATE_DIFFICULTY = 8
GAME_STATE_BEGIN_LEVEL = 9

GAME_DIFFICULTY_TOO_EASY = 0
GAME_DIFFICULTY_EASY = 1
GAME_DIFFICULTY_NORMAL = 2
GAME_DIFFICULTY_HARD = 3

show_debug_info = False
game_level = 1
total_levels = 3
game_state = GAME_STATE_MAIN_MENU
game_paused = False
game_difficulty = GAME_DIFFICULTY_HARD
map_grid = False
max_spawn = 20
max_mice = 120
total_spawn = 0
total_mice = 0
total_mice_caught = 0
total_mice_escaped = 0
mice_spawned = 0
mice_caught = 0
mice_roaming = 0
mice_safe = 0
mice_speed = 0

#-------------------------------------------------------------------------------
# Class for mouse spawn points
#-------------------------------------------------------------------------------
class SpawnPoint():
	position = Vector3()
	id = 0
	mice = 0
	time_next = 0
	time_between = 0
	difficulty = GAME_DIFFICULTY_EASY
	pass

#-------------------------------------------------------------------------------
# Class for mice
#-------------------------------------------------------------------------------
class Mouse():
	position = Vector3()
	destination = Vector3()
	active = False
	pass

#-------------------------------------------------------------------------------
# Setup main window
#-------------------------------------------------------------------------------
window_width, window_height = 800, 600
window_title = "Cat vs Mice"

try:
    # Try and create a window with multisampling (antialiasing)
    config = Config(sample_buffers=1, samples=4, 
                    depth_size=24, double_buffer=True)
    window = pyglet.window.Window(window_width, window_height, resizable=True,
                                  caption=window_title, config=config)
except pyglet.window.NoSuchConfigException:
    # Fall back to no multisampling for old hardware
	window = pyglet.window.Window(resizable=True, caption=window_title)
	print "Window using safe configuration"

#-------------------------------------------------------------------------------
# Window resizing
#-------------------------------------------------------------------------------
@window.event
def on_resize(width, height):
	glViewport(0, 0, width, height)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluPerspective(45.0, width / float(height), 0.1, 100.0)
	glMatrixMode(GL_MODELVIEW)
	return pyglet.event.EVENT_HANDLED

@window.event
def on_draw():
	global camera, fps, map, tabby, game_level
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

	if (game_state == GAME_STATE_PLAY_LEVEL or
		game_state == GAME_STATE_BEGIN_LEVEL):
		camera.update()
		map.draw_world()
		tabby.draw()
		draw_mice()

	if game_state == GAME_STATE_BEGIN_LEVEL:
		for i in range(total_spawn):
			x = spawn[i].position.x
			z = spawn[i].position.z
			glColor3f(1.0, 0.0, 0.0)
			glLineWidth(6.0)
			glBegin(GL_LINES)
			glVertex3f(x, 0.0, z)
			glVertex3f(x, 0.5, z)
			glEnd()

	# Text rendering
	glMatrixMode(GL_PROJECTION)
	glPushMatrix()
	glLoadIdentity()
	glOrtho(0, window.width, 0, window.height, -1, 1)
	glMatrixMode(GL_MODELVIEW)
	glPushMatrix()
	glLoadIdentity()
	if show_debug_info:
		fps.draw()
	if game_paused:
		label = pyglet.text.Label('Paused',
		                          font_name = None,
								  font_size = 25,
								  color = (255, 255, 0, 255),
								  x = window.width//2,
								  y = window.height//2,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
	if game_state == GAME_STATE_PLAY_LEVEL:
		label = pyglet.text.Label('Mice = %d / %d' % (mice_roaming, total_mice),
		                          font_name = None,
								  font_size = 14,
								  color = (255, 255, 0, 255),
								  x = window.width - 130,
								  y = window.height - 20,
								  anchor_x = 'left',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Caught = %d' % mice_caught,
		                          font_name = None,
								  font_size = 14,
								  color = (255, 255, 0, 255),
								  x = window.width - 130,
								  y = window.height - 40,
								  anchor_x = 'left',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Escaped = %d' % mice_safe,
		                          font_name = None,
								  font_size = 14,
								  color = (255, 255, 0, 255),
								  x = window.width - 130,
								  y = window.height - 60,
								  anchor_x = 'left',
								  anchor_y = 'center')
		label.draw()
	elif game_state == GAME_STATE_BEGIN_LEVEL:
		label = pyglet.text.Label('Level %d' % game_level,
		                          font_name = None,
								  font_size = 25,
								  color = (255, 255, 0, 255),
								  x = window.width//2,
								  y = window.height//2 + 25,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Get Ready',
		                          font_name = None,
								  font_size = 25,
								  color = (255, 255, 0, 255),
								  x = window.width//2,
								  y = window.height//2 - 25,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
	elif game_state == GAME_STATE_LEVEL_DONE:
		label = pyglet.text.Label('Level Completed!',
		                          font_name = None,
								  font_size = 25,
								  x = window.width//2,
								  y = window.height//2 + 50,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Mice caught = %d' % mice_caught,
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 10,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Mice escaped = %d' % mice_safe,
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 50,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Press SPACE to continue',
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 100,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
	elif game_state == GAME_STATE_FAIL:
		label = pyglet.text.Label('FAIL! Cat fell into water ... Game Over',
		                          font_name = None,
								  font_size = 25,
								  x = window.width//2,
								  y = window.height//2 + 50,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Total mice caught = %d' % total_mice_caught,
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 10,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Total mice escaped = %d' % total_mice_escaped,
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 50,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Press SPACE to continue',
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 100,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
	elif game_state == GAME_STATE_GAME_DONE:
		label = pyglet.text.Label('Game Over',
		                          font_name = None,
								  font_size = 25,
								  x = window.width//2,
								  y = window.height//2 + 50,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Total mice caught = %d' % total_mice_caught,
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 10,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Total mice escaped = %d' % total_mice_escaped,
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 50,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('Press SPACE to continue',
		                          font_name = None,
								  font_size = 20,
								  x = window.width//2,
								  y = window.height//2 - 100,
								  anchor_x = 'center',
								  anchor_y = 'center')
		label.draw()
	elif game_state == GAME_STATE_MAIN_MENU:
		label = pyglet.text.Label('CAT vs MICE',
		                          font_name = None,
								  font_size = 30,
								  x = 20,
								  y = window.height - 50,
								  anchor_x = 'left',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('(1) Play',
		                          font_name = None,
								  font_size = 20,
								  x = 35,
								  y = window.height - 200,
								  anchor_x = 'left',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('(2) Instructions',
		                          font_name = None,
								  font_size = 20,
								  x = 35,
								  y = window.height - 240,
								  anchor_x = 'left',
								  anchor_y = 'center')
		label.draw()
		label = pyglet.text.Label('(3) Exit',
		                          font_name = None,
								  font_size = 20,
								  x = 35,
								  y = window.height - 280,
								  anchor_x = 'left',
								  anchor_y = 'center')
		label.draw()
	elif game_state == GAME_STATE_INSTRUCTIONS:
		label = pyglet.text.Label('GOAL: Use the cat to catch the mice before they',
		                          font_name = None,
								  font_size = 18,
								  x = 35,
								  y = window.height - 100,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('             reach the safety of the garden shed.',
		                          font_name = None,
								  font_size = 18,
								  x = 35,
								  y = window.height - 130,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('KEYBOARD:',
		                          font_name = None,
								  font_size = 18,
								  x = 35,
								  y = window.height - 170,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('     Arrow Keys ... Move the cat',
		                          font_name = None,
								  font_size = 18,
								  x = 55,
								  y = window.height - 200,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('     p ... Pause the game',
		                          font_name = None,
								  font_size = 18,
								  x = 55,
								  y = window.height - 230,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('     + / - ... Zoom camera in and out',
		                          font_name = None,
								  font_size = 18,
								  x = 55,
								  y = window.height - 260,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('     Page Up / Page Down ... Tilt camera up and down',
		                          font_name = None,
								  font_size = 18,
								  x = 55,
								  y = window.height - 290,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('     Tab ... Toggle frames per second display',
		                          font_name = None,
								  font_size = 18,
								  x = 55,
								  y = window.height - 320,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
		label = pyglet.text.Label('Press SPACE to return to main menu.',
		                          font_name = None,
								  font_size = 18,
								  x = 35,
								  y = window.height - 390,
								  anchor_x = 'left',
								  anchor_y = 'center')        
		label.draw()
	glPopMatrix()
	glMatrixMode(GL_PROJECTION)
	glPopMatrix()
	glMatrixMode(GL_MODELVIEW)
	
#-------------------------------------------------------------------------------
# Setup OpenGL
#-------------------------------------------------------------------------------
def setup_opengl():
	glClearColor(0.3, 0.5, 0.7, 0.0)
	glEnable(GL_DEPTH_TEST)
	glDepthFunc(GL_LEQUAL)
	glShadeModel(GL_SMOOTH)
	#glEnable(GL_BLEND)
	#glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
	
#-------------------------------------------------------------------------------
# Add a spawn point for mice
#-------------------------------------------------------------------------------
def new_spawn(x, z, id, num_mice, time_next, time_between, difficulty):
	global spawn, total_mice, total_spawn
	spawn[total_spawn].position = Vector3(x, 0.0, z)
	spawn[total_spawn].id = id
	spawn[total_spawn].num_mice = num_mice
	spawn[total_spawn].time_next = time_next
	spawn[total_spawn].time_between = time_between
	spawn[total_spawn].difficulty = difficulty
	total_spawn += 1
	total_mice += num_mice

#-------------------------------------------------------------------------------
# Update all roaming mice
#-------------------------------------------------------------------------------
def update_mice():
	global map, mice, mice_caught, mice_roaming, mice_safe, mice_speed, mice_spawned
	global total_mice_escaped, total_mice_caught
	s = map.get_shed_position()
	c = tabby.get_position()
	for i in range(mice_spawned):
		if mice[i].active == True:
			# Is mouse home?
			if (abs(s.x - mice[i].position.x) < 0.25 and
			    abs(s.z - mice[i].position.z) < 0.25):
				mice[i].active = False
				mice_roaming -= 1
				mice_safe += 1
				total_mice_escaped += 1
			# Is mouse caught?
			elif (abs(c.x - mice[i].position.x) < 0.15 and
			      abs(c.z - mice[i].position.z) < 0.15):
				mice[i].active = False
				mice_roaming -= 1
				mice_caught += 1
				total_mice_caught += 1
			else:		
				# Has mouse reached destination
				if (abs(mice[i].position.x - mice[i].destination.x) < 0.2 and
				    abs(mice[i].position.z - mice[i].destination.z) < 0.2):
					dest_set = 0
					id = 0
					# Is destination a spawn point?
					for j in range(total_spawn):
						if spawn[j].position == mice[i].destination:
							id = spawn[j].id + 1
							#print 'Mouse %d reached spawn - heading for wp %d' % (i, id)
					# Is destination a waypoint?
					if id == 0:
						for j in range(map.get_num_waypoints()):
							if map.get_waypoint_position(j) == mice[i].destination:
								if map.is_waypoint_last(j) == True:
									mice[i].destination = s
									dest_set = 1
									#print 'Mouse %d heading for shed' % i
								else:
									id = map.get_waypoint_id(j) + 1
									#print 'Mouse %d heading for wp %d' % (i, id)
					# Find next waypoint
					if dest_set == 0:
						for j in range(map.get_num_waypoints()):
							if map.get_waypoint_id(j) == id:
								if dest_set == 0:
									mice[i].destination = map.get_waypoint_position(j)
									dest_set = 1
								else:
									if random.randint(1, 10) <= 5:
										mice[i].destination = map.get_waypoint_position(j)
				else:
					new_x = mice[i].position.x
					new_z = mice[i].position.z

					# Is cat nearby?
					d = math.sqrt(((new_x - c.x) * (new_x - c.x)) + ((new_z - c.z) * (new_z - c.z)))
					if d < 1.6:
						if new_x < c.x:
							new_x -= mice_speed
						elif new_x > c.x:
							new_x += mice_speed
						if new_z < c.z:
							new_z -= mice_speed
						elif new_z > c.z:
							new_z += mice_speed
					else:
						# Move mouse closer to next waypoint
						if new_x < mice[i].destination.x:
							new_x += mice_speed
						elif new_x > mice[i].destination.x:
							new_x -= mice_speed		
						if new_z < mice[i].destination.z:
							new_z += mice_speed
						elif new_z > mice[i].destination.z:
							new_z -= mice_speed
					
					if map.mice_collision_test(new_x, 0.0, new_z, 0.12) == True:
						new_x = mice[i].position.x
					if map.mice_collision_test(new_x, 0.0, new_z, 0.12) == True:
						new_z = mice[i].position.z
					mice[i].position.x = new_x
					mice[i].position.z = new_z
					
	
#-------------------------------------------------------------------------------
# Draw all roaming mice
#-------------------------------------------------------------------------------
def draw_mice():
	global mice, mice_spawned
	part = gluNewQuadric()
	for i in range(mice_spawned):
		if mice[i].active == True:
			glPushMatrix()
			glColor3f(0.5, 0.5, 0.5)
			glTranslatef(mice[i].position.x,
				         mice[i].position.y,
					     mice[i].position.z)
			gluSphere(part, 0.1, 10, 10)
			glPopMatrix()
	gluDeleteQuadric(part)
	
#-------------------------------------------------------------------------------
# Load level
#-------------------------------------------------------------------------------
def load_level(filename):
	global map, mice, mice_caught, mice_roaming, mice_safe, spawn, tabby, total_mice, total_spawn, mice_spawned
	map.reset()
	total_mice = 0
	total_spawn = 0
	mice_spawned = 0
	mice_caught = 0
	mice_roaming = 0
	mice_safe = 0
	envin = {}
		
	# Setup file parsing
	envin['tabby'] = lambda x,y,z: tabby.set_position(x,y,z)
	envin['fence'] = lambda x1,z1,x2,z2: map.new_fence(x1,z1,x2,z2)
	envin['wall'] = lambda x1,z1,x2,z2,r,g,b: map.new_wall(x1,z1,x2,z2,r,g,b)
	envin['window'] = lambda x1,z1,x2,z2,r,g,b: map.new_window(x1,z1,x2,z2,r,g,b)
	envin['door'] = lambda x1,z1,x2,z2,r,g,b: map.new_door(x1,z1,x2,z2,r,g,b)
	envin['slab'] = lambda x1,z1,x2,z2: map.new_slab(x1,z1,x2,z2)
	envin['fbox'] = lambda x1,z1,x2,z2,r,g,b: map.new_fbox(x1,z1,x2,z2,r,g,b)
	envin['bush'] = lambda x,z: map.new_bush(x, z)
	envin['pool'] = lambda x1,z1,x2,z2: map.new_pool(x1,z1,x2,z2)
	envin['shed'] = lambda x,z,d: map.new_shed(x,z,d)
	envin['spawn'] = lambda x,z,i,m,n,b,d: new_spawn(x,z,i,m,n,b,d)
	envin['waypoint'] = lambda x,z,i,l: map.new_waypoint(x,z,i,l)
	envin['quad'] = lambda x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4,r,g,b: map.new_quad(x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4,r,g,b)
	
	# Parse the input file
	parse_file(filename, envin)

#-------------------------------------------------------------------------------
# Update game
#-------------------------------------------------------------------------------
def update(dt):
	global game_paused, keys, mice, mice_roaming, mice_spawned, spawn, tabby, total_spawn
	global game_level, game_state, map, camera, total_mice_caught, total_mice_escaped
	global show_debug_info

	# Toggle debug info
	if keys[key.TAB]:
		if show_debug_info == True:
			show_debug_info = False
		else:
			show_debug_info = True

    # EPIC FAIL!
	if game_state == GAME_STATE_FAIL:
		if keys[key.SPACE]:
			game_state = GAME_STATE_MAIN_MENU

	#Instructions
	if game_state == GAME_STATE_INSTRUCTIONS:
		if keys[key.SPACE]:
			game_state = GAME_STATE_MAIN_MENU

	# Main menu
	if game_state == GAME_STATE_MAIN_MENU:
		if keys[key._1]:
			total_mice_caught = 0
			total_mice_escaped = 0
			game_level = 0
			game_state = GAME_STATE_LOAD_LEVEL
		elif keys[key._2]:
			game_state = GAME_STATE_INSTRUCTIONS
		elif keys[key._3]:
			sys.exit(0)

	# Level completed
	if game_state == GAME_STATE_LEVEL_DONE:
		if keys[key.SPACE]:
			game_state = GAME_STATE_LOAD_LEVEL

	# Game completed
	if game_state == GAME_STATE_GAME_DONE:
		if keys[key.SPACE]:
			game_state =GAME_STATE_MAIN_MENU

	# Begin a level
	if game_state == GAME_STATE_BEGIN_LEVEL:
		d = camera.get_distance()
		if d > 10.0:
			camera.zoom_in()
		else:
			game_state = GAME_STATE_PLAY_LEVEL

	# Load next level or game over
	if game_state == GAME_STATE_LOAD_LEVEL:
		game_level += 1
		if game_level > total_levels:
			game_state = GAME_STATE_GAME_DONE
		else:
			if game_level == 1:
				load_level('level01.txt')
			elif game_level == 2:
				load_level('level02.txt')
			elif game_level == 3:
				load_level('level03.txt')
			c = tabby.get_position()
			camera.set_viewpoint(c.x, c.y, c.z)
			camera.set_distance(35.0)
			game_state = GAME_STATE_BEGIN_LEVEL

	# Check for end of level
	if game_state == GAME_STATE_PLAY_LEVEL:
		if mice_roaming == 0 and mice_spawned == total_mice:
			game_state = GAME_STATE_LEVEL_DONE

	# Playing the level
	if game_state == GAME_STATE_PLAY_LEVEL:
		# Check for game pause/unpause
		if keys[key.P]:
			if game_paused == False:
				game_paused = True
			else:
				game_paused = False
		
		if game_paused == False:
			# Update mice spawners
			for i in range(total_spawn):
				if spawn[i].difficulty <= game_difficulty and spawn[i].num_mice > 0:
					spawn[i].time_next -= dt
					if spawn[i].time_next <= 0:
						mice[mice_spawned].position = Vector3(spawn[i].position.x,
								                              spawn[i].position.y,
															  spawn[i].position.z)
						mice[mice_spawned].destination = Vector3(spawn[i].position.x,
						                                         spawn[i].position.y,
																 spawn[i].position.z)
						mice[mice_spawned].active = True
						mice_spawned += 1
						mice_roaming += 1
						spawn[i].time_next = spawn[i].time_between
						spawn[i].num_mice -= 1
			
			# Update mice
			update_mice()
	
			# Cat movement
			cat_pos = tabby.get_position()
			# Is cat in the drink?
			if map.cat_water_test(cat_pos.x, cat_pos.y, cat_pos.z) == True:
				game_state = GAME_STATE_FAIL
			else:
				r = tabby.get_radius()
				new_x = cat_pos.x
				new_z = cat_pos.z
				if keys[key.UP]:
					new_z = cat_pos.z - tabby.get_speed()
					if (new_z < 0.0 or
						map.cat_collision_test(new_x, cat_pos.y, new_z, r + 0.02) == True):
						new_z = cat_pos.z
				elif keys[key.DOWN]:
					new_z = cat_pos.z + tabby.get_speed()
					if (new_z >= map.get_size() or
						map.cat_collision_test(new_x, cat_pos.y, new_z, r + 0.02) == True):
						new_z = cat_pos.z
				if keys[key.LEFT]:
					new_x = cat_pos.x - tabby.get_speed()
					if (new_x < 0.0 or
				        map.cat_collision_test(new_x, cat_pos.y, new_z, r + 0.02) == True):
						new_x = cat_pos.x
				elif keys[key.RIGHT]:
					new_x = cat_pos.x + tabby.get_speed()
					if (new_x >= map.get_size() or
						map.cat_collision_test(new_x, cat_pos.y, new_z, r + 0.02) == True):
						new_x = cat_pos.x
				tabby.set_position(new_x, cat_pos.y, new_z)
				camera.set_viewpoint(new_x, cat_pos.y, new_z)
	
		# Camera adjustments
		if (game_state != GAME_STATE_MAIN_MENU and
			game_state != GAME_STATE_INGAME_MENU):
			if keys[key.PAGEUP]:
				camera.rotate_up()
			elif keys[key.PAGEDOWN]:
				camera.rotate_down()
			if keys[key.EQUAL]:
				camera.zoom_in()
			elif keys[key.MINUS]:
				camera.zoom_out()
pyglet.clock.schedule_interval(update, 1/60.0)

#-------------------------------------------------------------------------------
# Start the game
#-------------------------------------------------------------------------------
def main():
	global camera, game_data, fps, keys, map, mice, mice_speed, spawn, tabby
	global game_difficulty
	
	random.seed()
	setup_opengl()
	fps = pyglet.clock.ClockDisplay(font=pyglet.font.load(None, 14),
		                            color=(2,2,0,1))
									
	# Setup keyboard handler
	keys = key.KeyStateHandler()
	window.push_handlers(keys)
	
	# Create game components
	camera = Camera()
	map = Map(map_grid)
	tabby = Cat()

	spawn = {}
	for i in range(max_spawn):
		spawn[i] = SpawnPoint()

	mice = {}
	for i in range(max_mice):
		mice[i] = Mouse()

	mice_speed = tabby.get_speed() - 0.05

	# Go! Go! Go!
	pyglet.app.run()

