from parsefile import *
from pyglet.gl import *
from vector import *
import pyglet
import math

PI_OVER_180 = (3.14159265 / 180.0)
FBOX = 0.49
BUSH = 0.3	
SHED = 0.75

class Map(object):
	def __init__(self, grid=False):
		self.size = 50
		self.grid = grid
		self.num_waypoints = 0

		# Map objects
		self.fences = []
		self.walls = []
		self.windows = []
		self.doors = []
		self.slabs = []
		self.fboxes = []
		self.bushes = []
		self.pools = []
		self.shed = Vector3()
		self.shed_door = 'E'
		self.waypoints = []
		self.quads = []

	def get_size(self):
		return self.size
		
	def get_shed_position(self):
		return self.shed

	def reset(self):
		self.fences[:] = []
		self.walls[:] = []
		self.windows[:] = []
		self.doors[:] = []
		self.slabs[:] = []
		self.fboxes[:] = []
		self.bushes[:] = []
		self.pools[:] = []
		self.waypoints[:] = []
		self.num_waypoints = 0
		self.quads[:] = []

	def new_fence(self, x1, z1, x2, z2):
		self.fences.append((x1, z1, x2, z2))

	def new_wall(self, x1, z1, x2, z2, r, g, b):
		self.walls.append((x1, z1, x2, z2, r, g, b))
		
	def new_window(self, x1, z1, x2, z2, r, g, b):
		self.windows.append((x1, z1, x2, z2, r, g, b))
		
	def new_door(self, x1, z1, x2, z2, r, g, b):
		self.doors.append((x1, z1, x2, z2, r, g, b))
		
	def new_slab(self, x1, z1, x2, z2):
		self.slabs.append((x1, z1, x2, z2))
		
	def new_fbox(self, x1, z1, x2, z2, r, g, b):
		self.fboxes.append((x1, z1, x2, z2, r, g, b))
		
	def new_bush(self, x, z):
		self.bushes.append((x, z))
		
	def new_pool(self, x1, z1, x2, z2):
		self.pools.append((x1, z1, x2, z2))
		self.new_slab(x1 - 1.0, z1 - 1.0, x2 + 1.0, z1)
		self.new_slab(x1 - 1.0, z1, x1, z2)
		self.new_slab(x2, z1, x2 + 1.0, z2)
		self.new_slab(x1 - 1.0, z2, x2 + 1.0, z2 + 1.0)
		
	def new_shed(self, x, z, d):
		self.shed.x = x
		self.shed.z = z
		if d == 'E' or d == 'W' or d == 'S':
			self.shed_door = d
		else:
			self.shed_door = 'E'

	def new_quad(self, x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4, r, g, b):
		self.quads.append((x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4, r, g, b))
			
	def new_waypoint(self, x, z, id, last):
		self.waypoints.append((x, z, id, last))
		self.num_waypoints += 1
		
	def get_num_waypoints(self):
		return self.num_waypoints
		
	def get_waypoint_position(self, i):
		return Vector3(self.waypoints[i][0], 0.0, self.waypoints[i][1])
		
	def get_waypoint_id(self, i):
		return self.waypoints[i][2]
		
	def is_waypoint_last(self, i):
		return self.waypoints[i][3]
			
	def draw_ground(self):
		glColor3f(0.4, 0.65, 0.4)
		glBegin(GL_QUADS)
		glVertex3f(0.0, 0.0, 0.0)
		glVertex3f(0.0, 0.0, self.size)
		glVertex3f(self.size, 0.0, self.size)
		glVertex3f(self.size, 0.0, 0.0)
		glEnd()

	def draw_fences(self, fences):
		for i in range(len(fences)):
			x1 = fences[i][0]
			z1 = fences[i][1]
			x2 = fences[i][2]
			z2 = fences[i][3]
			glColor3f(0.45, 0.45, 0.25)
			glBegin(GL_QUADS)
			glVertex3f(x1-0.01, 0.9, z1-0.01)
			glVertex3f(x2-0.01, 0.9, z2-0.01)
			glVertex3f(x2-0.01, 0.8, z2-0.01)
			glVertex3f(x1-0.01, 0.8, z1-0.01)
			glVertex3f(x1-0.01, 0.2, z1-0.01)
			glVertex3f(x2-0.01, 0.2, z2-0.01)
			glVertex3f(x2-0.01, 0.1, z2-0.01)
			glVertex3f(x1-0.01, 0.1, z1-0.01)
			glEnd()
			glColor3f(0.5, 0.5, 0.3)
			if z1 == z2:
				x = x1
				for j in range(int(x2 - x1)):
					for k in range(5):
						glBegin(GL_QUADS)
						glVertex3f(x + (k * 0.2), 1.0, z1)
						glVertex3f(x + (k * 0.2), 0.0, z1)
						glVertex3f(x + (k * 0.2) + 0.1, 0.0, z1)
						glVertex3f(x + (k * 0.2) + 0.1, 1.0, z1)
						glEnd()
					x += 1.0
			elif x1 == x2:
				z = z1
				for j in range(int(z2 - z1)):
					for k in range(5):
						glBegin(GL_QUADS)
						glVertex3f(x1, 1.0, z + (k * 0.2))
						glVertex3f(x1, 0.0, z + (k * 0.2))
						glVertex3f(x1, 0.0, z + (k * 0.2) + 0.1)
						glVertex3f(x1, 1.0, z + (k * 0.2) + 0.1)
						glEnd()
					z += 1.0
					
	def draw_walls(self, walls):
		for i in range(len(walls)):
			x1 = walls[i][0]
			z1 = walls[i][1]
			x2 = walls[i][2]
			z2 = walls[i][3]
			r = walls[i][4]
			g = walls[i][5]
			b = walls[i][6]
			glColor3f(r, g, b)
			glBegin(GL_QUADS)
			glVertex3f(x1, 2.0, z1)
			glVertex3f(x1, 0.0, z1)
			glVertex3f(x2, 0.0, z2)
			glVertex3f(x2, 2.0, z2)
			glEnd()
			
	def draw_windows(self, windows):
		for i in range(len(windows)):
			x1 = windows[i][0]
			z1 = windows[i][1]
			x2 = windows[i][2]
			z2 = windows[i][3]
			r = windows[i][4]
			g = windows[i][5]
			b = windows[i][6]
			glColor3f(r, g, b)
			if z1 == z2:
				glBegin(GL_QUADS)
				# Vertical posts
				glVertex3f(x1, 1.5, z1)
				glVertex3f(x1, 0.5, z1)
				glVertex3f(x1 + 0.2, 0.5, z1)
				glVertex3f(x1 + 0.2, 1.5, z1)
				glVertex3f(x2 - 0.2, 1.5, z2)
				glVertex3f(x2 - 0.2, 0.5, z2)
				glVertex3f(x2, 0.5, z2)
				glVertex3f(x2, 1.5, z2)
				# Glass
				glColor3f(0.7, 0.7, 0.7)
				glVertex3f(x1 + 0.2, 1.4, z1)
				glVertex3f(x2 - 0.2, 1.4, z2)
				glVertex3f(x2 - 0.2, 0.6, z2)
				glVertex3f(x1 + 0.2, 0.6, z1)
				glEnd()
			elif x1 == x2:
				glBegin(GL_QUADS)
				# Vertical posts
				glVertex3f(x1, 1.5, z1)
				glVertex3f(x1, 0.5, z1)
				glVertex3f(x1, 0.5, z1 + 0.2)
				glVertex3f(x1, 1.5, z1 + 0.2)
				glVertex3f(x2, 1.5, z2 - 0.2)
				glVertex3f(x2, 0.5, z2 - 0.2)
				glVertex3f(x2, 0.5, z2)
				glVertex3f(x2, 1.5, z2)
				# Glass
				glColor3f(0.7, 0.7, 0.7)
				glVertex3f(x1, 1.4, z1 + 0.2)
				glVertex3f(x2, 1.4, z2 - 0.2)
				glVertex3f(x2, 0.6, z2 - 0.2)
				glVertex3f(x1, 0.6, z1 + 0.2)
				glEnd()
			# Horizontal posts
			glColor3f(r, g, b)
			glBegin(GL_QUADS)
			glVertex3f(x1, 1.5, z1)
			glVertex3f(x2, 1.5, z2)
			glVertex3f(x2, 1.4, z2)
			glVertex3f(x1, 1.4, z1)
			glVertex3f(x1, 0.6, z1)
			glVertex3f(x2, 0.6, z2)
			glVertex3f(x2, 0.5, z2)
			glVertex3f(x1, 0.5, z1)
			glEnd()

	def draw_doors(self, doors):
		for i in range(len(doors)):
			x1 = doors[i][0]
			z1 = doors[i][1]
			x2 = doors[i][2]
			z2 = doors[i][3]
			r = doors[i][4]
			g = doors[i][5]
			b = doors[i][6]
			glColor3f(r, g, b)
			glBegin(GL_QUADS)
			glVertex3f(x1, 1.5, z1)
			glVertex3f(x2, 1.5, z2)
			glVertex3f(x2, 0.0, z2)
			glVertex3f(x1, 0.0, z1)
			glEnd()
			
	def draw_slabs(self, slabs):
		for i in range(len(slabs)):
			x1 = slabs[i][0]
			z1 = slabs[i][1]
			x2 = slabs[i][2]
			z2 = slabs[i][3]
			glColor3f(0.7, 0.7, 0.7)
			glBegin(GL_QUADS)
			glVertex3f(x1, 0.02, z1)
			glVertex3f(x1, 0.02, z2)
			glVertex3f(x2, 0.02, z2)
			glVertex3f(x2, 0.02, z1)
			glColor3f(0.3, 0.3, 0.3)
			glVertex3f(x1, 0.02, z1)
			glVertex3f(x1, 0.0, z1)
			glVertex3f(x1, 0.0, z2)
			glVertex3f(x1, 0.02, z2)
			glVertex3f(x1, 0.02, z2)
			glVertex3f(x1, 0.0, z2)
			glVertex3f(x2, 0.0, z2)
			glVertex3f(x2, 0.02, z2)
			glVertex3f(x2, 0.02, z2)
			glVertex3f(x2, 0.0, z2)
			glVertex3f(x2, 0.0, z1)
			glVertex3f(x2, 0.02, z1)
			glEnd()
			
	def draw_fboxes(self, fboxes):
		for i in range(len(fboxes)):
			x1 = fboxes[i][0]
			z1 = fboxes[i][1]
			x2 = fboxes[i][2]
			z2 = fboxes[i][3]
			r = fboxes[i][4]
			g = fboxes[i][5]
			b = fboxes[i][6]
			glColor3f(r, g, b)
			if z1 == z2:
				# Box
				glBegin(GL_QUADS)
				glVertex3f(x1, 0.3, z1 - FBOX)
				glVertex3f(x1, 0.0, z1 - FBOX)
				glVertex3f(x1, 0.0, z1 + FBOX)
				glVertex3f(x1, 0.3, z1 + FBOX)
				glVertex3f(x1, 0.3, z1 + FBOX)
				glVertex3f(x1, 0.0, z1 + FBOX)
				glVertex3f(x2, 0.0, z1 + FBOX)
				glVertex3f(x2, 0.3, z1 + FBOX)
				glVertex3f(x2, 0.3, z1 + FBOX)
				glVertex3f(x2, 0.0, z1 + FBOX)
				glVertex3f(x2, 0.0, z1 - FBOX)
				glVertex3f(x2, 0.3, z1 - FBOX)
				glVertex3f(x2, 0.3, z1 - FBOX)
				glVertex3f(x2, 0.0, z1 - FBOX)
				glVertex3f(x1, 0.0, z1 - FBOX)
				glVertex3f(x1, 0.3, z1 - FBOX)
				glColor3f(0.1, 0.0, 0.0)
				glVertex3f(x1, 0.2, z1 - FBOX)
				glVertex3f(x1, 0.2, z1 + FBOX)
				glVertex3f(x2, 0.2, z1 + FBOX)
				glVertex3f(x2, 0.2, z1 - FBOX)
				glEnd()
				# Flowers
				x = x1
				glColor3f(0.0, 0.5, 0.0)
				glLineWidth(4.0)
				for j in range(int(x2 - x1)):
					glBegin(GL_LINES)
					if j != 0 and j != int(x2 - x1):
						glVertex3f(x + j, 0.2, z1)
						glVertex3f(x + j, 0.5, z1)
					glVertex3f(x + j + 0.5, 0.2, z1)
					glVertex3f(x + j + 0.5, 0.5, z1)
					glEnd()
			elif x1 == x2:
				# Box
				glBegin(GL_QUADS)
				glVertex3f(x1 - FBOX, 0.3, z1)
				glVertex3f(x1 - FBOX, 0.0, z1)
				glVertex3f(x1 - FBOX, 0.0, z2)
				glVertex3f(x1 - FBOX, 0.3, z2)
				glVertex3f(x1 - FBOX, 0.3, z2)
				glVertex3f(x1 - FBOX, 0.0, z2)
				glVertex3f(x1 + FBOX, 0.0, z2)
				glVertex3f(x1 + FBOX, 0.3, z2)
				glVertex3f(x1 + FBOX, 0.3, z2)
				glVertex3f(x1 + FBOX, 0.0, z2)
				glVertex3f(x1 + FBOX, 0.0, z1)
				glVertex3f(x1 + FBOX, 0.3, z1)
				glVertex3f(x1 + FBOX, 0.3, z1)
				glVertex3f(x1 + FBOX, 0.0, z1)
				glVertex3f(x1 - FBOX, 0.0, z1)
				glVertex3f(x1 - FBOX, 0.3, z1)
				glColor3f(0.1, 0.0, 0.0)
				glVertex3f(x1 - FBOX, 0.2, z1)
				glVertex3f(x1 - FBOX, 0.2, z2)
				glVertex3f(x1 + FBOX, 0.2, z2)
				glVertex3f(x1 + FBOX, 0.2, z1)
				glEnd()
				# Flowers
				z = z1
				glColor3f(0.0, 0.5, 0.0)
				glLineWidth(4.0)
				for j in range(int(z2 - z1)):
					glBegin(GL_LINES)
					if j != 0 and j != int(z2 - z1):
						glVertex3f(x1, 0.2, z + j)
						glVertex3f(x1, 0.5, z + j)
					glVertex3f(x1, 0.2, z + j + 0.5)
					glVertex3f(x1, 0.5, z + j + 0.5)
					glEnd()		
					
	def draw_bushes(self, bushes):
		for i in range(len(bushes)):
			x = bushes[i][0]
			z = bushes[i][1]
			glColor3f(0.2, 0.0, 0.0)
			glLineWidth(4.0)
			glBegin(GL_LINES)
			glVertex3f(x, 0.0, z)
			glVertex3f(x, 0.15, z)
			glEnd()
			glColor3f(0.1, 0.4, 0.1)
			glBegin(GL_TRIANGLES)
			glVertex3f(x, 0.15, z)
			glVertex3f(x - BUSH, 0.5, z - BUSH)
			glVertex3f(x - BUSH, 0.5, z + BUSH)
			glVertex3f(x, 0.15, z)
			glVertex3f(x - BUSH, 0.5, z + BUSH)
			glVertex3f(x + BUSH, 0.5, z + BUSH)
			glVertex3f(x, 0.15, z)
			glVertex3f(x + BUSH, 0.5, z - BUSH)
			glVertex3f(x + BUSH, 0.5, z + BUSH)
			glColor3f(0.2, 0.5, 0.2)
			glVertex3f(x - BUSH, 0.5, z - BUSH)
			glVertex3f(x - BUSH, 0.5, z + BUSH)
			glVertex3f(x, 0.7, z)
			glVertex3f(x + BUSH, 0.5, z - BUSH)
			glVertex3f(x + BUSH, 0.5, z + BUSH)
			glVertex3f(x, 0.7, z)
			glColor3f(0.3, 0.6, 0.3)
			glVertex3f(x - BUSH, 0.5, z + BUSH)
			glVertex3f(x + BUSH, 0.5, z + BUSH)
			glVertex3f(x, 0.7, z)
			glVertex3f(x - BUSH, 0.5, z - BUSH)
			glVertex3f(x + BUSH, 0.5, z - BUSH)
			glVertex3f(x, 0.7, z)
			glEnd()

	def draw_pools(self, pools):
		for i in range(len(pools)):
			x1 = pools[i][0]
			z1 = pools[i][1]
			x2 = pools[i][2]
			z2 = pools[i][3]
			glColor3f(0.5, 0.8, 1.0)
			glBegin(GL_QUADS)
			glVertex3f(x1, 0.01, z1)
			glVertex3f(x1, 0.01, z2)
			glVertex3f(x2, 0.01, z2)
			glVertex3f(x2, 0.01, z1)
			glEnd()
	
	def draw_shed(self):
		x = self.shed.x
		z = self.shed.z
		r = SHED
		br = 0.25
		h = 0.2
		glBegin(GL_QUADS)
		# Bottom
		glColor3f(0.1, 0.2, 0.0)
		glVertex3f(x - r, 0.01, z - r)
		glVertex3f(x + r, 0.01, z - r)
		glVertex3f(x + r, 0.01, z + r)
		glVertex3f(x - r, 0.01, z + r)
		# South wall
		glColor3f(0.6, 0.6, 0.3)
		glVertex3f(x - r, 2.0, z + r)
		glVertex3f(x - r, h, z + r)
		glVertex3f(x + r, h, z + r)
		glVertex3f(x + r, 2.0, z + r)
		# East wall
		glColor3f(0.5, 0.5, 0.2)
		glVertex3f(x + r, 2.0, z + r)
		glVertex3f(x + r, h, z + r)
		glVertex3f(x + r, h, z - r)
		glVertex3f(x + r, 2.0, z - r)
		# West wall
		glVertex3f(x - r, 2.0, z - r)
		glVertex3f(x - r, h, z - r)
		glVertex3f(x - r, h, z + r)
		glVertex3f(x - r, 2.0, z + r)
		# Roof
		glColor3f(0.4, 0.4, 0.4)
		glVertex3f(x - r - 0.1, 2.0, z - r - 0.1)
		glVertex3f(x + r + 0.1, 2.0, z - r - 0.1)
		glVertex3f(x + r + 0.1, 2.0, z + r + 0.1)
		glVertex3f(x - r - 0.1, 2.0, z + r + 0.1)
		# Blocks
		glColor3f(0.4, 0.4, 0.4)
		glVertex3f(x - r, h, z + r - br)
		glVertex3f(x - r, 0.0, z + r - br)
		glVertex3f(x - r, 0.0, z + r)
		glVertex3f(x - r, h, z + r)	
		glVertex3f(x - r + br, h, z + r)
		glVertex3f(x - r + br, 0.0, z + r)
		glVertex3f(x - r + br, 0.0, z + r - br)
		glVertex3f(x - r + br, h, z + r - br)
		
		glVertex3f(x + r - br, h, z + r - br)
		glVertex3f(x + r - br, 0.0, z + r - br)
		glVertex3f(x + r - br, 0.0, z + r)
		glVertex3f(x + r - br, h, z + r)
		glVertex3f(x + r, h, z + r)
		glVertex3f(x + r, 0.0, z + r)
		glVertex3f(x + r, 0.0, z + r - br)
		glVertex3f(x + r, h, z + r - br)
		
		glVertex3f(x - r, h, z - r)
		glVertex3f(x - r, 0.0, z - r)
		glVertex3f(x - r, 0.0, z - r + br)
		glVertex3f(x - r, h, z - r + br)
		glVertex3f(x - r, h, z - r + br)
		glVertex3f(x - r, 0.0, z - r + br)
		glVertex3f(x - r + br, 0.0, z - r + br)
		glVertex3f(x - r + br, h, z - r + br)
		
		glVertex3f(x + r - br, h, z - r + br)
		glVertex3f(x + r - br, 0.0, z - r + br)
		glVertex3f(x + r, 0.0, z - r + br)
		glVertex3f(x + r, h, z - r + br)
		glVertex3f(x + r, h, z - r + br)
		glVertex3f(x + r, 0.0, z - r + br)
		glVertex3f(x + r, 0.0, z - r)
		glVertex3f(x + r, h, z - r)

		glColor3f(0.6, 0.6, 0.6)
		glVertex3f(x - r, h, z + r)
		glVertex3f(x - r, 0.0, z + r)
		glVertex3f(x - r + br, 0.0, z + r)
		glVertex3f(x - r + br, h, z + r)
		glVertex3f(x + r - br, h, z + r)
		glVertex3f(x + r - br, 0.0, z + r)
		glVertex3f(x + r, 0.0, z + r)
		glVertex3f(x + r, h, z + r)
		
		# Door
		glColor3f(0.3, 0.3, 0.0)
		if self.shed_door == "E":
			glVertex3f(x + r + 0.01, 1.5 + h, z + 0.5)
			glVertex3f(x + r + 0.01, h, z + 0.5)
			glVertex3f(x + r + 0.01, h, z - 0.5)
			glVertex3f(x + r + 0.01, 1.5 + h, z - 0.5)
		elif self.shed_door == 'S':
			glVertex3f(x - 0.5, 1.5, z + r + 0.01)
			glVertex3f(x - 0.5, h, z + r + 0.01)
			glVertex3f(x + 0.5, h, z + r + 0.01)
			glVertex3f(x + 0.5, 1.5, z + r + 0.01)
		elif self.shed_door == 'W':
			glVertex3f(x - r - 0.01, 1.5, z + 0.5)
			glVertex3f(x - r - 0.01, h, z + 0.5)
			glVertex3f(x - r - 0.01, h, z - 0.5)
			glVertex3f(x - r - 0.01, 1.5, z - 0.5)
		glEnd()

	def draw_quads(self, quads):
		for i in range(len(quads)):
			x1 = quads[i][0]
			y1 = quads[i][1]
			z1 = quads[i][2]
			x2 = quads[i][3]
			y2 = quads[i][4]
			z2 = quads[i][5]
			x3 = quads[i][6]
			y3 = quads[i][7]
			z3 = quads[i][8]
			x4 = quads[i][9]
			y4 = quads[i][10]
			z4 = quads[i][11]
			r = quads[i][12]
			g = quads[i][13]
			b = quads[i][14]
			glColor3f(r, g, b)
			glBegin(GL_QUADS)
			glVertex3f(x1, y1, z1)
			glVertex3f(x2, y2, z2)
			glVertex3f(x3, y3, z3)
			glVertex3f(x4, y4, z4)
			glEnd()
		
	def sphere_line_test(self, p1, p2, p3, r):
		dp = Vector3()
		dp.x = p2.x - p1.x
		dp.y = p2.y - p1.y
		dp.z = p2.z - p1.z
		a = dp.x * dp.x + dp.y * dp.y + dp.z * dp.z
		b = 2 * (dp.x * (p1.x - p3.x) + dp.y * (p1.y - p3.y) + dp.z * (p1.z - p3.z))
		c = p3.x * p3.x + p3.y * p3.y + p3.z * p3.z
		c += p1.x * p1.x + p1.y * p1.y + p1.z * p1.z
		c -= 2 * (p3.x * p1.x + p3.y * p1.y + p3.z * p1.z)
		c -= r * r
		t = b * b - 4 * a * c
		if t < 0:
			return False
		else:
			numer = ((p3.x - p1.x) * (p2.x - p1.x) + (p3.y - p1.y) * (p2.y - p1.y) + (p3.z - p1.z) * (p2.z - p1.z))
			denom = ((p2.x - p1.x) * (p2.x - p1.x) + (p2.y - p1.y) * (p2.y - p1.y) + (p2.z - p1.z) * (p2.z - p1.z))
			if denom == 0:
				return False
			u = numer / denom
			if u > 0 and u < 1:
				return True
			else:
				return False

	def cat_water_test(self, x, y, z):
		for i in range(len(self.pools)):
			x1 = self.pools[i][0]
			z1 = self.pools[i][1]
			x2 = self.pools[i][2]
			z2 = self.pools[i][3]
			if (x1 < x and x < x2 and z1 < z and z < z2):
				return True
		return False
			
	def cat_collision_test(self, x, y, z, r):
		p3 = Vector3(x, y, z)
		# Walls
		for i in range(len(self.walls)):
			p1 = Vector3(self.walls[i][0], 0.0, self.walls[i][1])
			p2 = Vector3(self.walls[i][2], 0.0, self.walls[i][3])
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
				
		# Fences
		for i in range(len(self.fences)):
			p1 = Vector3(self.fences[i][0], 0.0, self.fences[i][1])
			p2 = Vector3(self.fences[i][2], 0.0, self.fences[i][3])
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
				
		# Flower boxes
		for i in range(len(self.fboxes)):
			x1 = self.fboxes[i][0]
			z1 = self.fboxes[i][1]
			x2 = self.fboxes[i][2]
			z2 = self.fboxes[i][3]
			# North side
			if z1 == z2:
				p1 = Vector3(x1, 0.0, z1 - FBOX)
				p2 = Vector3(x2, 0.0, z1 - FBOX)
			else:
				p1 = Vector3(x1 - FBOX, 0.0, z1)
				p2 = Vector3(x1 + FBOX, 0.0, z1)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# West side
			if z1 == z2:
				p1 = Vector3(x1, 0.0, z1 - FBOX)
				p2 = Vector3(x1, 0.0, z1 + FBOX)
			else:
				p1 = Vector3(x1 - FBOX, 0.0, z1)
				p1 = Vector3(x2 - FBOX, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# South side
			if z1 == z2:
				p1 = Vector3(x1, 0.0, z1 + FBOX)
				p2 = Vector3(x2, 0.0, z2 + FBOX)
			else:
				p1 = Vector3(x2 - FBOX, 0.0, z2)
				p2 = Vector3(x2 + FBOX, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# East side
			if z1 == z2:
				p1 = Vector3(x2, 0.0, z1 - FBOX)
				p2 = Vector3(x2, 0.0, z2 + FBOX)
			else:
				p1 = Vector3(x1 + FBOX, 0.0, z1)
				p2 = Vector3(x2 + FBOX, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
		
		# Bushes
		for i in range(len(self.bushes)):
			x = self.bushes[i][0]
			z = self.bushes[i][1]
			# North side
			p1 = Vector3(x - BUSH, 0.0, z - BUSH)
			p2 = Vector3(x + BUSH, 0.0, z - BUSH)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# West side
			p1 = Vector3(x - BUSH, 0.0, z - BUSH)
			p2 = Vector3(x - BUSH, 0.0, z + BUSH)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# South side
			p1 = Vector3(x - BUSH, 0.0, z + BUSH)
			p2 = Vector3(x + BUSH, 0.0, z + BUSH)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# East side
			p1 = Vector3(x + BUSH, 0.0, z - BUSH)
			p2 = Vector3(x + BUSH, 0.0, z + BUSH)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True

		# Shed
		x = self.shed.x
		z = self.shed.z
		# North side
		p1 = Vector3(x - SHED, 0.0, z - SHED)
		p2 = Vector3(x + SHED, 0.0, z - SHED)
		if self.sphere_line_test(p1, p2, p3, r) == True:
			return True
		# West side
		p1 = Vector3(x - SHED, 0.0, z - SHED)
		p2 = Vector3(x - SHED, 0.0, z + SHED)
		if self.sphere_line_test(p1, p2, p3, r) == True:
			return True
		# South side
		p1 = Vector3(x - SHED, 0.0, z + SHED)
		p2 = Vector3(x + SHED, 0.0, z + SHED)
		if self.sphere_line_test(p1, p2, p3, r) == True:
			return True
		# East side
		p1 = Vector3(x + SHED, 0.0, z - SHED)
		p2 = Vector3(x + SHED, 0.0, z + SHED)
		if self.sphere_line_test(p1, p2, p3, r) == True:
			return True
				
		return False
		
	def mice_collision_test(self, x, y, z, r):
		p3 = Vector3(x, y, z)
		# Walls
		for i in range(len(self.walls)):
			p1 = Vector3(self.walls[i][0], 0.0, self.walls[i][1])
			p2 = Vector3(self.walls[i][2], 0.0, self.walls[i][3])
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True

		# Flower boxes
		for i in range(len(self.fboxes)):
			x1 = self.fboxes[i][0]
			z1 = self.fboxes[i][1]
			x2 = self.fboxes[i][2]
			z2 = self.fboxes[i][3]
			# North side
			if z1 == z2:
				p1 = Vector3(x1, 0.0, z1 - FBOX)
				p2 = Vector3(x2, 0.0, z1 - FBOX)
			else:
				p1 = Vector3(x1 - FBOX, 0.0, z1)
				p2 = Vector3(x1 + FBOX, 0.0, z1)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# West side
			if z1 == z2:
				p1 = Vector3(x1, 0.0, z1 - FBOX)
				p2 = Vector3(x1, 0.0, z1 + FBOX)
			else:
				p1 = Vector3(x1 - FBOX, 0.0, z1)
				p1 = Vector3(x2 - FBOX, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# South side
			if z1 == z2:
				p1 = Vector3(x1, 0.0, z1 + FBOX)
				p2 = Vector3(x2, 0.0, z2 + FBOX)
			else:
				p1 = Vector3(x2 - FBOX, 0.0, z2)
				p2 = Vector3(x2 + FBOX, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# East side
			if z1 == z2:
				p1 = Vector3(x2, 0.0, z1 - FBOX)
				p2 = Vector3(x2, 0.0, z2 + FBOX)
			else:
				p1 = Vector3(x1 + FBOX, 0.0, z1)
				p2 = Vector3(x2 + FBOX, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
				
		# Pools
		for i in range(len(self.pools)):
			x1 = self.pools[i][0]
			z1 = self.pools[i][1]
			x2 = self.pools[i][2]
			z2 = self.pools[i][3]
			# North side
			p1 = Vector3(x1, 0.0, z1)
			p2 = Vector3(x2, 0.0, z1)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# West side
			p1 = Vector3(x1, 0.0, z1)
			p2 = Vector3(x1, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# South side
			p1 = Vector3(x1, 0.0, z2)
			p2 = Vector3(x2, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
			# East side
			p1 = Vector3(x2, 0.0, z1)
			p2 = Vector3(x2, 0.0, z2)
			if self.sphere_line_test(p1, p2, p3, r) == True:
				return True
				
		return False
		
	def draw_world(self):
		self.draw_ground()
		self.draw_fences(self.fences)
		self.draw_walls(self.walls)
		self.draw_windows(self.windows)
		self.draw_doors(self.doors)
		self.draw_slabs(self.slabs)
		self.draw_fboxes(self.fboxes)
		self.draw_bushes(self.bushes)
		self.draw_pools(self.pools)
		self.draw_shed()
		self.draw_quads(self.quads)

		# Draw reference grid
		if self.grid:
			y = 0.02
			glColor3f(0.3, 0.3, 0.1)
			glLineWidth(1.0)
			glBegin(GL_LINES)
			for i in range(0, self.size):
				glVertex3f(i, 0.0, 0.0)
				glVertex3f(i, 0.0, self.size)
				glVertex3f(0.0, 0.0, i)
				glVertex3f(self.size, 0.0, i)
			glEnd()
		glFlush()
