import math

class Vector3(object):
	__slots__ = ('x', 'y', 'z')
	
	def __init__(self, x=0.0, y=0.0, z=0.0):
		self.x = x
		self.y = y
		self.z = z
	
	def __str__(self):
		return '(%.1f, %.1f, %.1f)' % (self.x, self.y, self.z)

	def __repr__(self):
		return str(self)

	def __eq__(self, v):
		return self.x == v.x and self.y == v.y and self.z == v.z

	def __cmp__(self, v):
		return self.__eq__(v)

	def __ne__(self, v):
		return not self.__eq__(v)

	def __getitem__(self, i):
		if i == 0: return self.x
		if i == 1: return self.y
		if i == 2: return self.z
		return "Vector index out-of-range"

	def __setitem__(self, i, v):
		if i == 0: self.x = v
		if i == 1: self.y = v
		if i == 2: self.z = v

	def __add__(self, v):
		return Vector3(self.x + v.x, self.y + v.y, self.z + v.z)
		
	def __sub__(self, v):
		return Vector3(self.x - v.x, self.y - v.y, self.z - v.z)

	def length(self):
		return math.sqrt(self.dot(self))

	def normalize(self):
		len = self.length()
		if len == 0:
			len = 1
		self.x = self.x / len
		self.y = self.y / len
		self.z = self.z / len
		
	def cross(self, v):
		new_x = self.y * v.z - v.y * self.z
		new_y = v.x * self.z - self.x * v.z
		new_z = self.x * v.y - v.x * self.y
		self.x = new_x
		self.y = new_y
		self.z = new_z

	def dot(self, v):
		return self.x * v.x + self.y * v.y + self.z * v.z

	def scale(self, s):
		self.x = self.x * s
		self.y = self.y * s
		self.z = self.z * s

